package com.qmth.distributed.print;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.aiDataStatistics.Calculator;
import com.qmth.distributed.print.aiDataStatistics.SimHash;
import com.qmth.distributed.print.aiDataStatistics.dto.AiDataCountDto;
import com.qmth.distributed.print.aiDataStatistics.dto.AiDataDto;
import com.qmth.distributed.print.aiDataStatistics.dto.AiDataExportDto;
import com.qmth.distributed.print.aiDataStatistics.dto.AiDataReportDto;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.mark.entity.MarkOcrStudentQuestion;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.service.MarkOcrStudentQuestionService;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.mark.service.MarkTaskService;
import com.qmth.teachcloud.mark.utils.BigDecimalUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import javax.annotation.Resource;
import javax.naming.spi.DirObjectFactory;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collector;
import java.util.stream.Collectors;

/**
 * AI自动评卷样本数据分析
 */
@SpringBootTest
@RunWith(SpringRunner.class)
public class AiDataStatisticsTest {
    private final static Logger log = LoggerFactory.getLogger(AiDataStatisticsTest.class);
    @Resource
    private MarkOcrStudentQuestionService markOcrStudentQuestionService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkTaskService markTaskService;

    double fullScore = 15d;
    double rate = 0.2;
    Long examId = 657567737580617728l;
    Long questionId = 657572149854208079l;
    String paperNumber = "2025052000001";

    @Test
    public void test() {
        try {
            FileInputStream inputStream = new FileInputStream("/Users/xiaofei/qmth/ai/毛泽东思想和中国特色社会主义理论体系概论-ai成绩.xlsx");
            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, inputStream, 0);
            List<AiDataDto> dataDtoList = excelReader.getObjectList(AiDataDto.class);

            List<MarkStudent> markStudentList = listMarkStudent(examId, paperNumber);
            Map<String, Long> studentMap = markStudentList.stream().collect(Collectors.toMap(x -> x.getStudentCode(), y -> y.getId()));

            List<MarkOcrStudentQuestion> markOcrStudentQuestionListAli = listMarkOcrStudentQuestion(examId, paperNumber, questionId);
            Map<Long, String> markOcrStudentQuestionListAliMap = markOcrStudentQuestionListAli.stream().collect(Collectors.toMap(x -> x.getStudentId(), y -> y.getOcrContent()));

            for (AiDataDto aiDataDto : dataDtoList) {
                Long studentId = studentMap.get(aiDataDto.getStudentCode());
                if (markOcrStudentQuestionListAliMap.containsKey(studentId)) {
                    aiDataDto.setBaiduOcrResult(markOcrStudentQuestionListAliMap.get(studentId));
                }
            }

            // 导出审核样本
//            exportSampleData(dataDtoList);

            // 导出统计报告
            exportReportData(dataDtoList);

        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
    }

    private void exportReportData(List<AiDataDto> dataDtoList) {
        List<AiDataReportDto> list = new ArrayList<>();
        AiDataReportDto aiDataReportDto = new AiDataReportDto();
        // 满分
        aiDataReportDto.setFullScore(fullScore);
        // 样本数
        aiDataReportDto.setSampleCount(dataDtoList.size());
        // 人评均分
        aiDataReportDto.setPersonAvgScore(dataDtoList.stream().collect(Collectors.averagingDouble(AiDataDto::getPersonScore)).doubleValue());
        //一致率分值
        aiDataReportDto.setConsistentScore(fullScore * rate);
        //有效样本数
        long count = dataDtoList.stream().filter(m -> StringUtils.isNotBlank(m.getBaiduOcrResult())).count();
        aiDataReportDto.setEffectiveSampleCount(Math.toIntExact(count));
        //相关系数
        aiDataReportDto.setRelateCoefficient(calcCorrelation(dataDtoList));
        //机评均分
        aiDataReportDto.setAiAvgScore(dataDtoList.stream().collect(Collectors.averagingDouble(AiDataDto::getAiThirdScore)).doubleValue());
        //均分差值
        aiDataReportDto.setAvgDiv(aiDataReportDto.getPersonAvgScore() - aiDataReportDto.getAiAvgScore());
        // 一致率样本数(小于3分)
        long countConsistent = dataDtoList.stream().filter(m -> Math.abs(m.getThirdDiv()) < aiDataReportDto.getConsistentScore()).count();
        aiDataReportDto.setConsistentSampleCount(Math.toIntExact(countConsistent));
        // 一致率
        aiDataReportDto.setConsistentRate(BigDecimalUtils.div(aiDataReportDto.getConsistentSampleCount(), aiDataReportDto.getEffectiveSampleCount(), 3));
        // 人评一分一段
        aiDataReportDto.setPersonInterval(statisticsScoreInterval("person", dataDtoList));
        // 机评一分一段
        aiDataReportDto.setAiInterval(statisticsScoreInterval("ai", dataDtoList));
        list.add(aiDataReportDto);

        try {
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
            writer.writeObjects("报告", null, AiDataReportDto.class, list.listIterator());
            FileOutputStream fileOut = new FileOutputStream("/Users/xiaofei/qmth/ai/报告2.xlsx");
            writer.output(fileOut);
            fileOut.flush();
            fileOut.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private String statisticsScoreInterval(String type, List<AiDataDto> dataDtoList) {
        dataDtoList = dataDtoList.stream().filter(m -> StringUtils.isNotBlank(m.getBaiduOcrResult())).collect(Collectors.toList());
        int totalSize = dataDtoList.size();
        List<String> list = new ArrayList<>();
        Map<Double, Long> collect = null;
        if ("person".equals(type)) {
            collect = dataDtoList.stream().collect(Collectors.groupingBy(AiDataDto::getPersonScore, Collectors.counting()));
        } else if ("ai".equals(type)) {
            collect = dataDtoList.stream().collect(Collectors.groupingBy(AiDataDto::getAiThirdScore, Collectors.counting()));
        } else {
            throw new RuntimeException("类型错误");
        }

        List<AiDataCountDto> aiDataCountDtos = new ArrayList<>();
        for (Map.Entry<Double, Long> entry : collect.entrySet()) {
            AiDataCountDto aiDataCountDto = new AiDataCountDto();
            aiDataCountDto.setScore(entry.getKey());
            aiDataCountDto.setCount(Math.toIntExact(entry.getValue()));
            aiDataCountDtos.add(aiDataCountDto);
        }
        aiDataCountDtos.sort(Comparator.comparing(AiDataCountDto::getScore));

        int initScore = 0;
        while (initScore < 15) {
            String str = "(" + initScore + "," + (initScore + 1) + "]";
            int finalInitScore = initScore;
            int sum = aiDataCountDtos.stream().filter(m -> m.getScore() > finalInitScore && m.getScore() <= finalInitScore + 1).mapToInt(AiDataCountDto::getCount).sum();
            double div = BigDecimalUtils.div(sum, totalSize, 4);
            double mul = BigDecimalUtils.mul(div, 100);
            String s = String.format("%s    %d    %s", str, sum, mul + "%");
            list.add(s);
            initScore++;
        }
//        for (AiDataCountDto dto : aiDataCountDtos) {
//            double div = BigDecimalUtils.div(dto.getCount(), totalSize, 4);
//            double mul = BigDecimalUtils.mul(div, 100);
//            String s = String.format("%s    %d    %s", dto.getScore(), dto.getCount(), mul + "%");
//            list.add(s);
//        }
        return list.stream().collect(Collectors.joining("\r\n"));
    }

    private Double calcCorrelation(List<AiDataDto> dataDtoList) {
        List<Double> personScoreList = new ArrayList<>();
        List<Double> aiScoreList = new ArrayList<>();
        for (AiDataDto t : dataDtoList) {
            if (StringUtils.isBlank(t.getBaiduOcrResult())) {
                continue;
            }
            personScoreList.add(t.getPersonScore());
            aiScoreList.add(t.getAiThirdScore());
        }
        if (personScoreList.size() - aiScoreList.size() != 0) {
            throw new RuntimeException("分数不一致");
        }
        double[] doubles = personScoreList.stream().mapToDouble(Double::doubleValue).toArray();
        double[] doubles1 = aiScoreList.stream().mapToDouble(Double::doubleValue).toArray();
        return Calculator.correlation(doubles, doubles1);
    }

    private void exportSampleData(List<AiDataDto> dataDtoList) {
        dataDtoList = dataDtoList.stream().filter(m -> StringUtils.isNotBlank(m.getBaiduOcrResult())).collect(Collectors.toList());
        // 查找统计样本
        List<AiDataDto> sampleList = getSampleList(dataDtoList);

        List<AiDataExportDto> list = new ArrayList<>();
        for (AiDataDto ai1 : sampleList) {
            for (AiDataDto ai2 : dataDtoList) {
                if (ai1.getStudentCode().equals(ai2.getStudentCode())) {
                    continue;
                }
                int hmDistance = hmDistance(ai1.getBaiduOcrResult(), ai2.getBaiduOcrResult());
                double distance = StringUtils.getJaroWinklerDistance(ai1.getBaiduOcrResult(), ai2.getBaiduOcrResult());
                log.info("学号{}、{}，识别结果重复率：{}。", ai1.getStudentCode(), ai2.getStudentCode(), hmDistance);
                if ((hmDistance <= 3 || distance > 0.9)
                        && Math.abs(ai1.getAiThirdScore() - ai2.getAiThirdScore()) < 2
                        && Math.abs(ai1.getPersonScore() - ai2.getPersonScore()) > 3) {
                    AiDataExportDto dto = new AiDataExportDto();
                    dto.setStudentCode1(ai1.getStudentCode());
                    dto.setStudentName1(ai1.getStudentName());
                    dto.setStudentCode2(ai2.getStudentCode());
                    dto.setStudentName2(ai2.getStudentName());
                    dto.setPersonScore1(ai1.getPersonScore());
                    dto.setPersonScore2(ai2.getPersonScore());
                    dto.setAiScore1(ai1.getAiThirdScore());
                    dto.setAiScore2(ai2.getAiThirdScore());
                    dto.setOcrDiv(hmDistance);
                    dto.setDistance(distance);
                    dto.setOcrResult1(ai1.getBaiduOcrResult());
                    dto.setOcrResult2(ai2.getBaiduOcrResult());
                    list.add(dto);
                }
            }
        }


        try {
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
            writer.writeObjects("成绩单", null, AiDataExportDto.class, list.listIterator());
            FileOutputStream fileOut = new FileOutputStream("/Users/xiaofei/qmth/ai/12.xlsx");
            writer.output(fileOut);
            fileOut.flush();
            fileOut.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private List<AiDataDto> getSampleList(List<AiDataDto> dataDtoList) {
        return dataDtoList.stream().filter(m -> Math.abs(m.getThirdDiv()) >= 10).collect(Collectors.toList());
    }

    private List<MarkStudent> listMarkStudent(Long examId, String paperNumber) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId)
                .eq(MarkStudent::getPaperNumber, paperNumber);
        return markStudentService.list(queryWrapper);
    }

    private List<MarkOcrStudentQuestion> listMarkOcrStudentQuestion(Long examId, String paperNumber, Long questionId) {
        QueryWrapper<MarkOcrStudentQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkOcrStudentQuestion::getExamId, examId)
                .eq(MarkOcrStudentQuestion::getPaperNumber, paperNumber)
                .eq(MarkOcrStudentQuestion::getQuestionId, questionId);
        return markOcrStudentQuestionService.list(queryWrapper);
    }

    private List<Long> listMarkTask(Long examId, String paperNumber) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().select(MarkTask::getStudentId).eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber).eq(MarkTask::getStatus, MarkTaskStatus.MARKED);
        List<MarkTask> markTasks = markTaskService.list(queryWrapper);
        return markTasks.stream().map(MarkTask::getStudentId).collect(Collectors.toList());
    }

    private int hmDistance(String ocr1, String ocr2) {
        BigInteger hash1 = SimHash.hash(ocr1);
        BigInteger hash2 = SimHash.hash(ocr2);
        System.out.println("hash1：" + hash1);
        System.out.println("hash2：" + hash2);
        return SimHash.hmDistance(hash1, hash2);
    }

    public static void main(String[] args) {
        String str1 = "请使用签字笔作答(超出答题框,答题无效)\n" +
                "四、论述题(本大题共2小题,每小题15分,共30分)\n" +
                "1、\n" +
                "握新形势下坚持和运用好毛泽东思想,就要深刻认识和把握毛泽东思想活\n" +
                "的灵魂,即实事求是,群众路线和独立自主\n" +
                "1、.坚持实事求是,要求深入实际了解事物在面貌,为把握内在必然联系,按客观\n" +
                "规律办事;坚持实事求是,要清晰认识和正确把握我国的基本国情;坚持实事求\n" +
                "是,要不断推进实际践基础上的理论创新。\n" +
                "2.必须坚持群众路线,这要求我们要坚持人民是推动历史发展的根本力量;始终\n" +
                "坚持全心全意为人民服务的根本宗旨,始终保持党与人民群众的血肉联系\n" +
                "3.必须坚持独立自主,始终坚持中国的事情由中国人自己作决定自己来处理,其次\n" +
                "要始终坚持独立自主的和平外交战略,坚持走和平发展道路";
        String str2 = "请使用签字笔作答(超出答题框,答题无效)\n" +
                "四、论述题(本大题共2小题,每小题15分,共30分)\n" +
                "1.答:(1)坚持将毛泽东思想活的灵魂运用到当今的社会主义现代化建设。\n" +
                "毛泽东思想的话的灵魂包括实事求是、群众路线、独立自主。\n" +
                "①实事求是,指的是要一切从实际出发,理论联系实际,在实践中检验和发展真理。这是毛\n" +
                "娱东思想的精髓,也是我们党的思想路线。在新时代,仍要坚持实事求是,就是深入\n" +
                "实际了解事物本来面貌,把握事物内在本质联系,按客观规律做事;坚持实事求是,就是\n" +
                "清醒认识和把握正确我国基本目情;坚持实事求是在实践基础不断推进理论创新。\n" +
                "②群众路线是指一切众为了群众,一切依靠群众,从群众中来,到群众中去,把党的正确主\n" +
                "张转化群众的自觉行动。在新时代,坚持群众路线,就要坚持人民群众是推动历史发展\n" +
                "的根本力量;坚持全心全意为人民服务的根本宗旨;坚持党与群众间的血肉联系;坚持\n" +
                "让群众来评判党的工作。\n" +
                "③独立自立是指独立思考走自己的路;坚决卫国家主权,民族独立,在把立足点放在依\n" +
                "靠自己力量的基础上同时积极寻求外援,开展国际经济文化交流,学习吸收一切对我们有\n" +
                "益外目先进事物。坚持独立自主就要中国的事情由中国人民自己作主;坚持走中国特色\n" +
                "社会主义道路,坚持实行和平外交政策,走和平发展道路,为广大发展中国家提供力所能及的\n" +
                "帮助\n" +
                "(2)坚持“两个结合”,结合当今形势和需要,对毛泽东思想进行创造性运用,运用创新\n" +
                "型转化,推进马克思主义中化时代化,使其焕发出新的生机与活力。";
        BigInteger hash1 = SimHash.hash(str1);
        BigInteger hash2 = SimHash.hash(str2);
        System.out.println("hash1：" + hash1);
        System.out.println("hash2：" + hash2);
        System.out.println("海明距离：" + SimHash.hmDistance(hash1, hash2));


        System.out.println(StringUtils.getJaroWinklerDistance(str1, str2));
    }


}
