package com.qmth.distributed.print;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreModel;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreRequest;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreResult;
import com.qmth.boot.core.ai.model.llm.score.StandardAnswer;
import com.qmth.boot.core.ai.service.AiService;
import com.qmth.boot.tools.io.IOUtils;
import com.qmth.teachcloud.common.bean.dto.excel.BasicCourseImportDto;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.ExcelUtil;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionLevel;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionParam;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionPoint;
import com.qmth.teachcloud.mark.entity.MarkOcrStudentQuestion;
import com.qmth.teachcloud.mark.enums.AiQuestionParamModeStatus;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.AiUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 导入导出工具类
 * @Author: CaoZixuan
 * @Date: 2021-09-27
 */
@SpringBootTest
@RunWith(SpringRunner.class)
public class AiMarkTest {

    @Resource
    private DictionaryConfig dictionaryConfig;
    @Resource
    private MarkOcrStudentQuestionService markOcrStudentQuestionService;
    @Resource
    private MarkAiQuestionLevelService markAiQuestionLevelService;
    @Resource
    private MarkAiQuestionPointService markAiQuestionPointService;
    @Resource
    private MarkAiQuestionParamService markAiQuestionParamService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private AiService aiService;


    @Test
    public void test() throws Exception {
        Long examId = 664876885490008064l;
        // 空白卷有分的考生
        Long questionId = Long.parseLong("1934559380958556162");
        Long studentId = Long.parseLong("667325662096261120"); //667325662096261120 667325662343725056   667325664495403008
        String paperNumber = "27160014";
        String subjectName = "习近平新时代中国特色社会主义思想概论";

        // 无法评卷考生
//        Long questionId = Long.parseLong("1934063467957837828");
//        Long studentId = Long.parseLong("665624496190783488");
//        String paperNumber = "27160011";
//        String subjectName = "形势与政策（理论）";

        // code=400, body={"code":400000,"error":"request parameter invalid"}
//        Long questionId = Long.parseLong("1934063467957837827");
//        Long studentId = Long.parseLong("665624448732233728");
//        String paperNumber = "27160011";
//        String subjectName = "形势与政策（理论）";
        List<MarkOcrStudentQuestion> markOcrStudentQuestions = markOcrStudentQuestionService.listByStudentIdAndQuestionId(studentId, questionId);
        if (CollectionUtils.isEmpty(markOcrStudentQuestions)) {
            return;
        }
        String studentQuestionAnswer = this.mergeQuestionOcrResult(markOcrStudentQuestions);

        MarkAiQuestionParam markAiQuestionParam = markAiQuestionParamService.getByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId);
        List<StandardAnswer> standardAnswerList = buildStandardAnswer(markAiQuestionParam);

        MarkQuestion markQuestion = markQuestionService.getById(questionId);

        AutoScoreRequest request = new AutoScoreRequest();
        request.setSubjectName(subjectName);
        request.setScoreModel(AutoScoreModel.valueOf(markAiQuestionParam.getMode().name()));
        request.setTotalScore(markQuestion.getTotalScore());
        request.setIntervalScore(markAiQuestionParam.getMinScore());
        request.setQuestionBody(markAiQuestionParam.getMainTitle());
        request.setStudentAnswer(studentQuestionAnswer);
        request.setStandardAnswer(standardAnswerList);
        String aiUrl = "http://192.168.10.83:8090";
        // AI评卷
        try {
            AutoScoreResult autoScoreResult = aiService.autoScore(aiUrl, AiUtil.signatureOcr(dictionaryConfig.sysDomain()), request);
            System.out.println(JSON.toJSONString(autoScoreResult));
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new RuntimeException(e.getMessage());
        }
    }

    private String mergeQuestionOcrResult(List<MarkOcrStudentQuestion> markOcrStudentQuestionList) {
        if (CollectionUtils.isEmpty(markOcrStudentQuestionList)) {
            return null;
        }
        return markOcrStudentQuestionList.stream().filter(m -> StringUtils.isNotBlank(m.getOcrContent())).map(MarkOcrStudentQuestion::getOcrContent).collect(Collectors.joining(""));
    }

    private List<StandardAnswer> buildStandardAnswer(MarkAiQuestionParam markAiQuestionParam) {
        List<StandardAnswer> standardAnswerList = new ArrayList<>();
        // 得分点
        if (AiQuestionParamModeStatus.POINT.equals(markAiQuestionParam.getMode())) {
            List<MarkAiQuestionPoint> markAiQuestionPoints = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
            if (CollectionUtils.isEmpty(markAiQuestionPoints)) {
                throw ExceptionResultEnum.ERROR.exception("未设置得分点");
            }
            standardAnswerList = markAiQuestionPoints.stream().map(m -> {
                StandardAnswer standardAnswer = new StandardAnswer();
                standardAnswer.setScore(m.getScore());
                standardAnswer.setContent(m.getAnswer());
                return standardAnswer;
            }).collect(Collectors.toList());
        }
        // 档次
        else if (AiQuestionParamModeStatus.LEVEL.equals(markAiQuestionParam.getMode())) {
            List<MarkAiQuestionLevel> markAiQuestionLevels = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());
            if (CollectionUtils.isEmpty(markAiQuestionLevels)) {
                throw ExceptionResultEnum.ERROR.exception("未设置档次");
            }
            standardAnswerList = markAiQuestionLevels.stream().map(m -> {
                StandardAnswer standardAnswer = new StandardAnswer();
                standardAnswer.setLowScore(m.getMinScore());
                standardAnswer.setHighScore(m.getMaxScore());
                standardAnswer.setContent(m.getAnswer());
                return standardAnswer;
            }).collect(Collectors.toList());
        } else {
            throw ExceptionResultEnum.ERROR.exception("评分模式不存在");
        }
        return standardAnswerList;
    }
}
