package com.qmth.teachcloud.common.enums;

import com.qmth.teachcloud.common.contant.SystemConstant;

/**
 * @Description: pdf格式 enum
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/7
 */
public enum PageSizeEnum {
    A3("A3", 1191.0F,842.0F,  420, 297),
    A4("A4", 595.0F, 842.0F, 210, 297),
    A5("A5", 420.0F, 595.0F, 148, 210),
    B3("B3", 1417.0F,1000.0F,  500, 353),
    B4("B4", 1000.0F,708.0F,  353, 250),
    B5("B5", 498.0F, 708.0F, 176, 250),
    K8("8K", 1043.0F,737.0F,  390, 270);

    private String pageSize;

    private float width;

    private float height;

    // 单位mm
    private float pageWidth;
    // 单位mm
    private float pageHeight;

    private PageSizeEnum(String pageSize, float width, float height, float pageWidth, float pageHeight) {
        this.pageSize = pageSize;
        this.width = width;
        this.height = height;
        this.pageWidth = pageWidth;
        this.pageHeight = pageHeight;
    }

    public String getPageSize() {
        return pageSize;
    }

    public float getWidth() {
        return width;
    }

    public float getHeight() {
        return height;
    }

    public float getPageWidth() {
        return pageWidth;
    }

    public float getPageHeight() {
        return pageHeight;
    }

    /**
     * 状态转换 toName
     *
     * @param width
     * @param height
     * @return
     */
    public static String convertToName(float width, float height) {
        float deviation = SystemConstant.PAPER_DEVIATION;
        for (PageSizeEnum e : PageSizeEnum.values()) {
            float standardWidth = e.getWidth();
            float standardHeight = e.getHeight();
            if (standardWidth - deviation <= width && width <= standardWidth + deviation && standardHeight - deviation <= height && height <= standardHeight + deviation) {
                return e.name();
            }
        }
        return null;
    }

    /**
     * 状态转换 toEnum
     *
     * @param width
     * @param height
     * @return
     */
    public static PageSizeEnum convertToEnum(float width, float height) {
        for (PageSizeEnum e : PageSizeEnum.values()) {
            float deviation = SystemConstant.PAPER_DEVIATION;
            if (PageSizeEnum.A3.equals(e)) {
                deviation = 2 * SystemConstant.PAPER_DEVIATION;
            }
            float standardWidth = e.getWidth();
            float standardHeight = e.getHeight();
            if (standardWidth - deviation <= width && width <= standardWidth + deviation && standardHeight - deviation <= height && height <= standardHeight + deviation) {
                return e;
            }
            if (standardWidth - deviation <= height && height <= standardWidth + deviation && standardHeight - deviation <= width && width <= standardHeight + deviation) {
//                throw ExceptionResultEnum.ERROR.exception("请检查上传的【"+ e.name() + "】纸张字体方向是否反了");
                return e;
            }
        }
        throw ExceptionResultEnum.ERROR.exception("pdf格式不符合标准规范，请提供标准的pdf（可使用wps、office的word转换）");
    }

    public static PageSizeEnum matchPageSize(float width, float height) {
        float[] mSize = calMillimeter(width, height);
        if (mSize == null) {
            throw ExceptionResultEnum.ERROR.exception("PDF文件尺寸获取错误");
        }
        for (PageSizeEnum e : PageSizeEnum.values()) {
            if (mSize[0] >= calDeviation(e.getPageWidth(), -0.04f) && mSize[0] <= calDeviation(e.getPageWidth(), 0.04f)
                    && mSize[1] >= calDeviation(e.getPageHeight(), -0.04f) && mSize[1] <= calDeviation(e.getPageHeight(), 0.04f)) {
                return e;
            }
        }
        return null;
//        throw ExceptionResultEnum.ERROR.exception("PDF文件尺寸非系统允许尺寸，请提供标准的PDF（可使用WPS、office进行word转换）");
    }


    /**
     * 计算pdf的毫米尺寸
     *
     * @param width  DPI宽
     * @param height DPI高
     */
    private static float[] calMillimeter(float width, float height) {
        float mWidth = (float) (width / 72 * 2.54 * 10);
        float mHeight = (float) (height / 72 * 2.54 * 10);
        return new float[]{mWidth, mHeight};
    }

    /**
     * 计算允许的误差值
     *
     * @param value     值
     * @param deviation 偏差（可为负）
     * @return
     */
    private static float calDeviation(float value, float deviation) {
        return value * (1 + deviation);
    }
}
